@@ -18,7 +18,7 @@ from requests import HTTPError
 from tabulate import tabulate
 
 from datapane import __rev__, __version__
-from datapane.common import JDict, SDict, log
+from datapane.common import JDict, SDict, _setup_dp_logging, log
 
 from . import api
 from . import config as c
@@ -33,7 +33,8 @@ EXTRA_OUT: bool = False
 #  - convert to use typer (https://github.com/tiangolo/typer) or autoclick
 def init(verbosity: int, config_env: str):
     """Init the cmd-line env"""
-    api.init(config_env=config_env, verbosity=verbosity)
+    c.init(config_env=config_env)
+    _setup_dp_logging(verbosity=verbosity)
 
     # config_f = c.load_from_envfile(config_env)
     # _debug = debug if debug is not None else c.config.debug
@@ -130,7 +131,7 @@ def cli(ctx, verbose: int, env: str):
 def login(obj: DPContext, token, server):
     """Login to a server with the given API token."""
     config = c.Config(server=server, token=token)
-    r = api.check_login(config=config)
+    r = api.check_login(config=config, cli_login=True)
 
     # update config with valid values
     with c.update_config(obj.env) as x:
@@ -267,7 +268,7 @@ def deploy(name: Optional[str], script: Optional[str], config: Optional[str], vi
     log.debug(f"Packaging and uploading Datapane project {dp_cfg.name}")
 
     # start the build process
-    with click_spinner.spinner(), scripts.build_bundle(dp_cfg) as sdist:
+    with scripts.build_bundle(dp_cfg) as sdist:
 
         if EXTRA_OUT:
             tf: tarfile.TarFile
